import logging
import argparse

from response_measurer.post import Post
from response_measurer import version


def parse_args():
    # BASE PARSER
    argument_parser = argparse.ArgumentParser(description='A simple python HTTP request response measurer')
    argument_parser.add_argument('--host', dest='host', required=True, type=str,
                                 help='Enter a host')
    argument_parser.add_argument('--loop-count', dest='loop_count', required=False, type=int,
                                 default=1, help='Enter a loop count which repeats the requests given number'
                                                 'Default value is 1')
    argument_parser.add_argument('--timeout', dest='timeout', required=False, default=60,
                                 type=float, help='Enter a timeout. Default value is 60 seconds')
    argument_parser.add_argument('--log-level', dest='log_level', required=False, default='INFO', type=str,
                                 help='Define log level [INFO,DEBUG,WARN,WARNING,CRITICAL,ERROR,FATAL]. Default is INFO')
    argument_parser.add_argument('--version', action='version', version=version)

    # SUB PARSER
    sub_argument_parsers = argument_parser.add_subparsers(
        help='Enter request type. Supported requests: post', dest='request_type')

    # POST PARSER
    post_parser = sub_argument_parsers.add_parser('post')
    post_parser.add_argument('--data', dest='data', required=False, default='',
                             help='Define your data. Default is empty-string ""')

    # GEt PARSER
    get_parser = sub_argument_parsers.add_parser('get')

    args = vars(argument_parser.parse_args())
    return args


def run():
    parameters = parse_args()

    log_level = get_log_level(parameters["log_level"])

    # Configure logging
    logging.basicConfig(
        level=log_level,
        format='%(asctime)s.%(msecs)03d %(levelname)s %(module)s - %(funcName)s: %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S'
    )

    logging.info("response_measurer started")

    request_type = parameters["request_type"]
    if parameters["request_type"] == "post":
        logging.debug("POST request is running")
        post = Post(parameters)
        post.run()
    elif request_type == "get":
        logging.error("GET is not implemented yet")
        exit(1)


def get_log_level(_log_level: str):
    numeric_level = getattr(logging, _log_level.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError(f"Invalid log level: {_log_level}")
    return numeric_level


if __name__ == '__main__':
    run()



def calculate_mean(values: list):
    return sum(values) / len(values)


# Ref: https://stackoverflow.com/a/7464107/14091937
def calculate_percentile(n: list, p: float):
    """
    Find the percentile of a list of values

    @parameter N - A list of values.  N must be sorted.
    @parameter P - A float value from 0.0 to 1.0

    @return - The percentile of the values.
    """
    k = int(round(p * len(n) + 0.5))
    return n[k-1]



import requests
import logging
from response_measurer.calculaters import *


class Post:
    def __init__(self, parameters: dict):
        self.parameters = parameters

    def send_post_request(self):
        try:
            values_list = []
            for i in range(self.parameters["loop_count"]):
                response = requests.post(
                    self.parameters["host"],
                    data=self.parameters["data"],
                    timeout=self.parameters["timeout"]
                )
                values_list.append(response.elapsed.total_seconds())
            return values_list
        except Exception as e:
            logging.error(e)
            exit(1)

    def run(self):
        values = self.send_post_request()
        # Calculate mean
        mean = calculate_mean(values)
        logging.info(f"Mean of the POST requests: {mean:.5f}")
        # Percentile %50
        p50 = calculate_percentile(values, 0.5)
        logging.info(f"Percentile %50 of the POST requests: {p50:.5f}")
        p99 = calculate_percentile(values, 0.99)
        logging.info(f"Percentile %99 of the POST requests: {p99:.5f}")
